# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.bank_transaction_flow_direction import BankTransactionFlowDirection
from postfinancecheckout.models.bank_transaction_state import BankTransactionState
from postfinancecheckout.models.currency_bank_account import CurrencyBankAccount
from postfinancecheckout.models.payment_adjustment import PaymentAdjustment
from typing import Optional, Set
from typing_extensions import Self

class BankTransaction(BaseModel):
    """
    BankTransaction
    """ # noqa: E501
    adjustments: Optional[List[PaymentAdjustment]] = Field(default=None, description="Adjustments are changes made to the initial transaction amount, such as fees or corrections.")
    currency_bank_account: Optional[CurrencyBankAccount] = Field(default=None, alias="currencyBankAccount")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=100)]] = Field(default=None, description="A client generated nonce which identifies the entity to be created. Subsequent creation requests with the same external ID will not create new entities but return the initially created entity instead.", alias="externalId")
    posting_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The posting amount refers to the monetary value recorded for the bank transaction prior to any adjustments.", alias="postingAmount")
    source: Optional[StrictInt] = Field(default=None, description="The source indicates how the bank transaction was created.")
    value_date: Optional[datetime] = Field(default=None, description="The value date indicates the date on which the transaction amount becomes effective.", alias="valueDate")
    type: Optional[StrictInt] = Field(default=None, description="The bank transaction's type.")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    reference: Optional[StrictStr] = Field(default=None, description="A unique reference to identify the bank transaction.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    value_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The value amount represents the net monetary value of the transaction after applicable deductions.", alias="valueAmount")
    flow_direction: Optional[BankTransactionFlowDirection] = Field(default=None, alias="flowDirection")
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the bank transaction was created by.", alias="createdBy")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[BankTransactionState] = None
    payment_date: Optional[datetime] = Field(default=None, description="The payment date specifies the date on which the payment was processed.", alias="paymentDate")
    total_adjustment_amount_including_tax: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Represents the total value of all adjustments to the bank transaction, including tax.", alias="totalAdjustmentAmountIncludingTax")
    __properties: ClassVar[List[str]] = ["adjustments", "currencyBankAccount", "plannedPurgeDate", "externalId", "postingAmount", "source", "valueDate", "type", "createdOn", "version", "reference", "linkedSpaceId", "valueAmount", "flowDirection", "createdBy", "id", "state", "paymentDate", "totalAdjustmentAmountIncludingTax"]

    @field_validator('external_id')
    def external_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BankTransaction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "adjustments",
            "planned_purge_date",
            "external_id",
            "posting_amount",
            "source",
            "value_date",
            "type",
            "created_on",
            "version",
            "reference",
            "linked_space_id",
            "value_amount",
            "created_by",
            "id",
            "payment_date",
            "total_adjustment_amount_including_tax",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in adjustments (list)
        _items = []
        if self.adjustments:
            for _item_adjustments in self.adjustments:
                if _item_adjustments:
                    _items.append(_item_adjustments.to_dict())
            _dict['adjustments'] = _items
        # override the default output from pydantic by calling `to_dict()` of currency_bank_account
        if self.currency_bank_account:
            _dict['currencyBankAccount'] = self.currency_bank_account.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BankTransaction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "adjustments": [PaymentAdjustment.from_dict(_item) for _item in obj["adjustments"]] if obj.get("adjustments") is not None else None,
            "currencyBankAccount": CurrencyBankAccount.from_dict(obj["currencyBankAccount"]) if obj.get("currencyBankAccount") is not None else None,
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "postingAmount": obj.get("postingAmount"),
            "source": obj.get("source"),
            "valueDate": obj.get("valueDate"),
            "type": obj.get("type"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "reference": obj.get("reference"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "valueAmount": obj.get("valueAmount"),
            "flowDirection": obj.get("flowDirection"),
            "createdBy": obj.get("createdBy"),
            "id": obj.get("id"),
            "state": obj.get("state"),
            "paymentDate": obj.get("paymentDate"),
            "totalAdjustmentAmountIncludingTax": obj.get("totalAdjustmentAmountIncludingTax")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.payment_terminal_configuration_version import PaymentTerminalConfigurationVersion
from postfinancecheckout.models.payment_terminal_location_version import PaymentTerminalLocationVersion
from postfinancecheckout.models.payment_terminal_state import PaymentTerminalState
from postfinancecheckout.models.payment_terminal_type import PaymentTerminalType
from typing import Optional, Set
from typing_extensions import Self

class PaymentTerminal(BaseModel):
    """
    PaymentTerminal
    """ # noqa: E501
    identifier: Optional[StrictStr] = Field(default=None, description="The unique identifier of the terminal, that is displayed on the device.")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[StrictStr] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    type: Optional[PaymentTerminalType] = None
    device_name: Optional[StrictStr] = Field(default=None, description="The name of the device that is currently linked to the payment terminal.", alias="deviceName")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    device_serial_number: Optional[StrictStr] = Field(default=None, description="The serial number of the device that is currently linked to the payment terminal.", alias="deviceSerialNumber")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    configuration_version: Optional[PaymentTerminalConfigurationVersion] = Field(default=None, alias="configurationVersion")
    location_version: Optional[PaymentTerminalLocationVersion] = Field(default=None, alias="locationVersion")
    default_currency: Optional[StrictStr] = Field(default=None, description="The default currency of the terminal.", alias="defaultCurrency")
    name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The name used to identify the payment terminal.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[PaymentTerminalState] = None
    __properties: ClassVar[List[str]] = ["identifier", "plannedPurgeDate", "externalId", "type", "deviceName", "version", "deviceSerialNumber", "linkedSpaceId", "configurationVersion", "locationVersion", "defaultCurrency", "name", "id", "state"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentTerminal from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "identifier",
            "planned_purge_date",
            "external_id",
            "device_name",
            "version",
            "device_serial_number",
            "linked_space_id",
            "default_currency",
            "name",
            "id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of type
        if self.type:
            _dict['type'] = self.type.to_dict()
        # override the default output from pydantic by calling `to_dict()` of configuration_version
        if self.configuration_version:
            _dict['configurationVersion'] = self.configuration_version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of location_version
        if self.location_version:
            _dict['locationVersion'] = self.location_version.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentTerminal from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "identifier": obj.get("identifier"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "type": PaymentTerminalType.from_dict(obj["type"]) if obj.get("type") is not None else None,
            "deviceName": obj.get("deviceName"),
            "version": obj.get("version"),
            "deviceSerialNumber": obj.get("deviceSerialNumber"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "configurationVersion": PaymentTerminalConfigurationVersion.from_dict(obj["configurationVersion"]) if obj.get("configurationVersion") is not None else None,
            "locationVersion": PaymentTerminalLocationVersion.from_dict(obj["locationVersion"]) if obj.get("locationVersion") is not None else None,
            "defaultCurrency": obj.get("defaultCurrency"),
            "name": obj.get("name"),
            "id": obj.get("id"),
            "state": obj.get("state")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.address import Address
from postfinancecheckout.models.environment import Environment
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.transaction_completion import TransactionCompletion
from postfinancecheckout.models.transaction_invoice_state import TransactionInvoiceState
from typing import Optional, Set
from typing_extensions import Self

class TransactionInvoice(BaseModel):
    """
    TransactionInvoice
    """ # noqa: E501
    completion: Optional[TransactionCompletion] = None
    derecognized_on: Optional[datetime] = Field(default=None, description="The date and time when the invoice was derecognized, meaning it is no longer considered outstanding or valid in the system.", alias="derecognizedOn")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total sum of all line items on the invoice, including taxes.")
    due_on: Optional[datetime] = Field(default=None, description="The due date for payment of the invoice.", alias="dueOn")
    outstanding_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The remaining amount the buyer owes to the merchant. A negative value indicates the invoice has been overpaid.", alias="outstandingAmount")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=100)]] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    time_zone: Optional[StrictStr] = Field(default=None, description="The time zone that this object is associated with.", alias="timeZone")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    paid_on: Optional[datetime] = Field(default=None, description="The date and time when the invoice was recorded as paid. May differ from the actual payment date due to processing delays.", alias="paidOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    line_items: Optional[List[LineItem]] = Field(default=None, description="The invoiced line items that will appear on the invoice document.", alias="lineItems")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    environment: Optional[Environment] = None
    derecognized_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the invoice was derecognized by.", alias="derecognizedBy")
    billing_address: Optional[Address] = Field(default=None, alias="billingAddress")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[TransactionInvoiceState] = None
    linked_transaction: Optional[StrictInt] = Field(default=None, description="The payment transaction this object is linked to.", alias="linkedTransaction")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The portion of the invoiced amount that corresponds to taxes.", alias="taxAmount")
    merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the invoice.", alias="merchantReference")
    __properties: ClassVar[List[str]] = ["completion", "derecognizedOn", "amount", "dueOn", "outstandingAmount", "plannedPurgeDate", "externalId", "timeZone", "language", "spaceViewId", "createdOn", "paidOn", "version", "lineItems", "linkedSpaceId", "environment", "derecognizedBy", "billingAddress", "id", "state", "linkedTransaction", "taxAmount", "merchantReference"]

    @field_validator('external_id')
    def external_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    @field_validator('merchant_reference')
    def merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TransactionInvoice from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "derecognized_on",
            "amount",
            "due_on",
            "outstanding_amount",
            "planned_purge_date",
            "external_id",
            "time_zone",
            "language",
            "space_view_id",
            "created_on",
            "paid_on",
            "version",
            "line_items",
            "linked_space_id",
            "derecognized_by",
            "id",
            "linked_transaction",
            "tax_amount",
            "merchant_reference",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of completion
        if self.completion:
            _dict['completion'] = self.completion.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of billing_address
        if self.billing_address:
            _dict['billingAddress'] = self.billing_address.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TransactionInvoice from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "completion": TransactionCompletion.from_dict(obj["completion"]) if obj.get("completion") is not None else None,
            "derecognizedOn": obj.get("derecognizedOn"),
            "amount": obj.get("amount"),
            "dueOn": obj.get("dueOn"),
            "outstandingAmount": obj.get("outstandingAmount"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "timeZone": obj.get("timeZone"),
            "language": obj.get("language"),
            "spaceViewId": obj.get("spaceViewId"),
            "createdOn": obj.get("createdOn"),
            "paidOn": obj.get("paidOn"),
            "version": obj.get("version"),
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "environment": obj.get("environment"),
            "derecognizedBy": obj.get("derecognizedBy"),
            "billingAddress": Address.from_dict(obj["billingAddress"]) if obj.get("billingAddress") is not None else None,
            "id": obj.get("id"),
            "state": obj.get("state"),
            "linkedTransaction": obj.get("linkedTransaction"),
            "taxAmount": obj.get("taxAmount"),
            "merchantReference": obj.get("merchantReference")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.address_create import AddressCreate
from typing import Optional, Set
from typing_extensions import Self

class SubscriberUpdate(BaseModel):
    """
    A subscriber represents everyone who is subscribed to a product.
    """ # noqa: E501
    reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the subscriber.")
    additional_allowed_payment_method_configurations: Optional[List[StrictInt]] = Field(default=None, description="Allow the subscriber to use these payment methods even if subscription products do not accept them.", alias="additionalAllowedPaymentMethodConfigurations")
    meta_data: Optional[Dict[str, StrictStr]] = Field(default=None, description="Allow to store additional information about the object.", alias="metaData")
    email_address: Optional[Annotated[str, Field(strict=True, max_length=254)]] = Field(default=None, description="The email address that is used to communicate with the subscriber. There can be only one subscriber per space with the same email address.", alias="emailAddress")
    disallowed_payment_method_configurations: Optional[List[StrictInt]] = Field(default=None, description="Prevent the subscriber from using these payment methods even if subscription products do accept them.", alias="disallowedPaymentMethodConfigurations")
    description: Optional[Annotated[str, Field(strict=True, max_length=200)]] = Field(default=None, description="The description used to identify the subscriber.")
    shipping_address: Optional[AddressCreate] = Field(default=None, alias="shippingAddress")
    language: Optional[StrictStr] = Field(default=None, description="The language that is used when communicating with the subscriber via emails and documents.")
    billing_address: Optional[AddressCreate] = Field(default=None, alias="billingAddress")
    version: StrictInt = Field(description="The version number indicates the version of the entity. The version is incremented whenever the entity is changed.")
    __properties: ClassVar[List[str]] = ["reference", "additionalAllowedPaymentMethodConfigurations", "metaData", "emailAddress", "disallowedPaymentMethodConfigurations", "description", "shippingAddress", "language", "billingAddress", "version"]

    @field_validator('reference')
    def reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubscriberUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of shipping_address
        if self.shipping_address:
            _dict['shippingAddress'] = self.shipping_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_address
        if self.billing_address:
            _dict['billingAddress'] = self.billing_address.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubscriberUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "reference": obj.get("reference"),
            "additionalAllowedPaymentMethodConfigurations": obj.get("additionalAllowedPaymentMethodConfigurations"),
            "metaData": obj.get("metaData"),
            "emailAddress": obj.get("emailAddress"),
            "disallowedPaymentMethodConfigurations": obj.get("disallowedPaymentMethodConfigurations"),
            "description": obj.get("description"),
            "shippingAddress": AddressCreate.from_dict(obj["shippingAddress"]) if obj.get("shippingAddress") is not None else None,
            "language": obj.get("language"),
            "billingAddress": AddressCreate.from_dict(obj["billingAddress"]) if obj.get("billingAddress") is not None else None,
            "version": obj.get("version")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import unittest

from constants import API_CONFIG, SPACE_ID
from postfinancecheckout import Configuration
from postfinancecheckout.models import CreationEntityState, WebhookListenerCreate
from postfinancecheckout.service import WebhookListenersService, WebhookURLsService


class TestWebhookListenersApi(unittest.TestCase):
    """postfinancecheckout WebhookListeners Service tests"""

    WEBHOOK_LISTENER_ENTITY_ID = 1472041829003
    WEBHOOK_LISTENER_NAME = "Python Mock UnitTest Webhook Listener"

    def setUp(self):
        self.webhook_listeners_service = WebhookListenersService(API_CONFIG)
        self.webhook_urls_service = WebhookURLsService(API_CONFIG)

    def test_webhook_listener_create_and_delete(self):
        """test_webhook_listener_create_and_delete() should create and delete webhook listener"""

        webhook_url_search_response = self.webhook_urls_service.get_webhooks_urls_search(
            space=SPACE_ID,
            expand=["url", "group"],
            limit=1,
            order="id:DESC",
            query="name:\"Webhook Test URL Listener\"")
        self.assertEqual(first=len(webhook_url_search_response.data), second=1,
                         msg="The length of the searched webhook urls list should be 1.")

        webhook_url_id = webhook_url_search_response.data[0].id
        webhook_listener_create = self.webhook_listener_create(webhook_url_id)
        created_response = self.webhook_listeners_service.post_webhooks_listeners(
            SPACE_ID,
            webhook_listener_create,
            expand=["url", "group"])

        self.assertEqual(first=created_response.state, second=CreationEntityState.ACTIVE,
                         msg="State must be \"ACTIVE\".")
        self.assertEqual(first=created_response.linked_space_id, second=SPACE_ID, msg="Space id should match.")
        self.assertEqual(first=created_response.name, second=self.WEBHOOK_LISTENER_NAME, msg="Name should match.")
        self.assertEqual(first=created_response.entity, second=self.WEBHOOK_LISTENER_ENTITY_ID,
                         msg="Entity should match.")
        self.assertEqual(first=created_response.url.id, second=webhook_url_id, msg="URL id should match.")
        self.assertEqual(first=created_response.url.name, second="Webhook Test URL Listener",
                         msg="URL name should match.")

        self.webhook_listeners_service.delete_webhooks_listeners_id(created_response.id, SPACE_ID)

        get_deleted_response = self.webhook_listeners_service.get_webhooks_listeners_id(created_response.id, SPACE_ID)
        self.assertEqual(first=get_deleted_response.id, second=created_response.id, msg="Id should match.")
        self.assertEqual(first=get_deleted_response.state, second=CreationEntityState.DELETED,
                         msg="State must be \"DELETED\".")
        self.assertEqual(first=get_deleted_response.linked_space_id, second=SPACE_ID, msg="Space id should match.")
        self.assertEqual(first=get_deleted_response.name, second=self.WEBHOOK_LISTENER_NAME, msg="Name should match.")

    def webhook_listener_create(self, webhook_url_id):
        return WebhookListenerCreate(
            entity=self.WEBHOOK_LISTENER_ENTITY_ID,
            name=self.WEBHOOK_LISTENER_NAME,
            state=CreationEntityState.ACTIVE,
            url=webhook_url_id
        )

    def test_webhook_listeners_search_without_custom_timeout(self):
        """test_webhook_listeners_search_without_custom_timeout() should search webhook listener without custom timeout"""

        search_response = self.webhook_listeners_service.get_webhooks_listeners_search(
            space=SPACE_ID,
            expand=["url", "group"],
            limit=1,
            order="id:DESC",
            query="name:Test AND state:INACTIVE")

        self.assertEqual(first=len(search_response.data), second=1,
                         msg="The length of the searched webhook listeners list should be 1.")
        for webhook_listener in search_response.data:
            self.assertEqual(first="Test", second=webhook_listener.name,
                             msg=f"Expected name to be \"Test\", got {webhook_listener.name}.")
            self.assertEqual(first=SPACE_ID, second=webhook_listener.linked_space_id,
                             msg=f"Expected space ID: {SPACE_ID} does not match, got {webhook_listener.linked_space_id}.")

    def test_webhook_listeners_search_with_custom_timeout(self):
        """test_webhook_listeners_search_with_custom_timeout() should search webhook listener with custom timeout: 36 sec."""

        # Multiple independent Configuration instances
        # API_CONFIG and custom_configuration do not share state or configuration
        custom_configuration = Configuration(user_id=API_CONFIG.user_id,
                                             authentication_key=API_CONFIG.authentication_key,
                                             request_timeout=36)
        webhook_listeners_service_with_custom_timeout = WebhookListenersService(custom_configuration)

        search_response = webhook_listeners_service_with_custom_timeout.get_webhooks_listeners_search(
            space=SPACE_ID,
            expand=["url", "group"],
            limit=1,
            order="id:DESC",
            query="name:Test AND state:INACTIVE")

        self.assertEqual(first=len(search_response.data), second=1,
                         msg="The length of the searched webhook listeners list should be 1.")
        for webhook_listener in search_response.data:
            self.assertEqual(first="Test", second=webhook_listener.name,
                             msg=f"Expected name to be \"Test\", got {webhook_listener.name}.")
            self.assertEqual(first=SPACE_ID, second=webhook_listener.linked_space_id,
                             msg=f"Expected space ID: {SPACE_ID} does not match, got {webhook_listener.linked_space_id}.")



# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from setuptools import setup, find_packages  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools
NAME = "postfinancecheckout"
VERSION = "6.3.0"
PYTHON_REQUIRES = ">=3.9"
REQUIRES = [
    "pydantic >= 2.10.4",
    "PyJWT >= 2.10.1",
    "python-dateutil >= 2.9.0.post0",
    "typing-extensions >= 4.12.2",
    "urllib3 >= 2.3, < 3",
    "cryptography >= 2.0"
]

EXTRAS_REQUIRE = {
    "dev": [
        "flake8 >= 7.1.1",
        "mypy >= 1.14.1",
        "pytest >= 3.9",
        "tox >= 4.23.2",
        "types-python-dateutil >= 2.9.0.20241206"
    ]
}

setup(
    name=NAME,
    version=VERSION,
    description="PostFinance Checkout API",
    author="postfinancecheckout",
    author_email="team@openapitools.org",
    url="",
    keywords=["OpenAPI", "OpenAPI-Generator", "PostFinance Checkout API"],
    install_requires=REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    long_description_content_type='text/markdown',
    long_description="""\
    The PostFinance Checkout API allows to create integrations, retrieve data and automate workflows.
    """,  # noqa: E501
    package_data={"postfinancecheckout": ["py.typed"]},
    classifiers = [
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "Topic :: Office/Business :: Financial :: Point-Of-Sale"
    ]
)


# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class PaymentTerminalDccTransactionSum(BaseModel):
    """
    Represents the aggregated summary of Dynamic Currency Conversion (DCC) transactions grouped by brand and currency combinations in a transaction summary receipt.
    """ # noqa: E501
    transaction_currency: Optional[StrictStr] = Field(default=None, description="The original currency of the transactions before DCC conversion (typically the merchant's local currency).", alias="transactionCurrency")
    transaction_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total sum of all transactions in the original transaction currency (the amount in merchant's local currency before DCC conversion).", alias="transactionAmount")
    dcc_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total sum of all transactions in the converted DCC currency (the amount paid by customers in their chosen currency).", alias="dccAmount")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    transaction_count: Optional[StrictInt] = Field(default=None, description="The total count of DCC transactions processed for this specific brand and currency combination.", alias="transactionCount")
    dcc_currency: Optional[StrictStr] = Field(default=None, description="The converted currency used in DCC transactions (the currency chosen by the customer for payment).", alias="dccCurrency")
    brand: Optional[StrictStr] = Field(default=None, description="The payment brand for which these DCC transactions are summarized.")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    __properties: ClassVar[List[str]] = ["transactionCurrency", "transactionAmount", "dccAmount", "id", "transactionCount", "dccCurrency", "brand", "version"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentTerminalDccTransactionSum from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "transaction_currency",
            "transaction_amount",
            "dcc_amount",
            "id",
            "transaction_count",
            "dcc_currency",
            "brand",
            "version",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentTerminalDccTransactionSum from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "transactionCurrency": obj.get("transactionCurrency"),
            "transactionAmount": obj.get("transactionAmount"),
            "dccAmount": obj.get("dccAmount"),
            "id": obj.get("id"),
            "transactionCount": obj.get("transactionCount"),
            "dccCurrency": obj.get("dccCurrency"),
            "brand": obj.get("brand"),
            "version": obj.get("version")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.gender import Gender
from typing import Optional, Set
from typing_extensions import Self

class CustomerPostalAddressCreate(BaseModel):
    """
    CustomerPostalAddressCreate
    """ # noqa: E501
    country: Optional[StrictStr] = Field(default=None, description="The two-letter country code (ISO 3166 format).")
    mobile_phone_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The phone number of a mobile phone.", alias="mobilePhoneNumber")
    gender: Optional[Gender] = None
    organization_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The organization's name.", alias="organizationName")
    city: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The city, town or village.")
    commercial_register_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The commercial registration number of the organization.", alias="commercialRegisterNumber")
    social_security_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The social security number.", alias="socialSecurityNumber")
    given_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The given or first name.", alias="givenName")
    postcode: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The postal code, also known as ZIP, postcode, etc.")
    legal_organization_form: Optional[StrictInt] = Field(default=None, description="The legal form of the organization.", alias="legalOrganizationForm")
    sales_tax_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The sales tax number of the organization.", alias="salesTaxNumber")
    date_of_birth: Optional[date] = Field(default=None, description="The date of birth.", alias="dateOfBirth")
    dependent_locality: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The dependent locality which is a sub-division of the state.", alias="dependentLocality")
    email_address: Optional[Annotated[str, Field(strict=True, max_length=254)]] = Field(default=None, description="The email address.", alias="emailAddress")
    phone_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The phone number.", alias="phoneNumber")
    sorting_code: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The sorting code identifying the post office where the PO Box is located.", alias="sortingCode")
    street: Optional[Annotated[str, Field(strict=True, max_length=300)]] = Field(default=None, description="The street or PO Box.")
    family_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The family or last name.", alias="familyName")
    postal_state: Optional[StrictStr] = Field(default=None, description="The name of the region, typically a state, county, province or prefecture.", alias="postalState")
    salutation: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="The salutation e.g. Mrs, Mr, Dr.")
    __properties: ClassVar[List[str]] = ["country", "mobilePhoneNumber", "gender", "organizationName", "city", "commercialRegisterNumber", "socialSecurityNumber", "givenName", "postcode", "legalOrganizationForm", "salesTaxNumber", "dateOfBirth", "dependentLocality", "emailAddress", "phoneNumber", "sortingCode", "street", "familyName", "postalState", "salutation"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CustomerPostalAddressCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CustomerPostalAddressCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "country": obj.get("country"),
            "mobilePhoneNumber": obj.get("mobilePhoneNumber"),
            "gender": obj.get("gender"),
            "organizationName": obj.get("organizationName"),
            "city": obj.get("city"),
            "commercialRegisterNumber": obj.get("commercialRegisterNumber"),
            "socialSecurityNumber": obj.get("socialSecurityNumber"),
            "givenName": obj.get("givenName"),
            "postcode": obj.get("postcode"),
            "legalOrganizationForm": obj.get("legalOrganizationForm"),
            "salesTaxNumber": obj.get("salesTaxNumber"),
            "dateOfBirth": obj.get("dateOfBirth"),
            "dependentLocality": obj.get("dependentLocality"),
            "emailAddress": obj.get("emailAddress"),
            "phoneNumber": obj.get("phoneNumber"),
            "sortingCode": obj.get("sortingCode"),
            "street": obj.get("street"),
            "familyName": obj.get("familyName"),
            "postalState": obj.get("postalState"),
            "salutation": obj.get("salutation")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import unittest
import uuid

from constants import API_CONFIG, SPACE_ID, MOCK_CARD_DATA
from utils import Utils
from postfinancecheckout.service import TransactionsService, RefundsService
from postfinancecheckout.models import RefundCreate, RefundState, RefundType, TransactionCompletionState, TransactionState

class TestRefundsService(unittest.TestCase):
    """postfinancecheckout refunds service tests"""

    def setUp(self):
        self.refunds_service = RefundsService(API_CONFIG)
        self.transactions_service = TransactionsService(API_CONFIG)

    def test_refund_of_completed_transaction_should_work(self):
        """
        Refund of fulfilled transaction should be created successfully.
        """

        transaction = self._create(Utils.get_transaction_create_payload())

        processed_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        transaction_completion = self.transactions_service.post_payment_transactions_id_complete_online(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            transaction_completion.state,
            TransactionCompletionState.SUCCESSFUL,
            "Transaction completion state must be SUCCESSFUL"
        )

        read_transaction = self.transactions_service.get_payment_transactions_id(
            transaction.id, SPACE_ID
        )

        self.assertEqual(
            read_transaction.state,
            TransactionState.FULFILL,
            "Transaction state must be FULFILL"
        )

        refund_create = self._get_refund_create(transaction)

        refund = self.refunds_service.post_payment_refunds(
            SPACE_ID, refund_create
        )

        self.assertEqual(
            refund.state,
            RefundState.SUCCESSFUL,
            "Refund state must be SUCCESSFUL"
        )

    def test_read_should_return_refund_data(self):
        """
        Refund read should return valid data.
        """

        transaction = self._create(Utils.get_transaction_create_payload())

        self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction.id, SPACE_ID, MOCK_CARD_DATA
        )

        self.transactions_service.post_payment_transactions_id_complete_online(
            transaction.id, SPACE_ID
        )

        refund_create = self._get_refund_create(transaction)

        refund = self.refunds_service.post_payment_refunds(
            SPACE_ID, refund_create
        )

        self.assertEqual(
            refund.state,
            RefundState.SUCCESSFUL,
            "Refund state must be SUCCESSFUL"
        )

        read_refund = self.refunds_service.get_payment_refunds_id(
            refund.id, SPACE_ID
        )

        self.assertEqual(
            refund.id, read_refund.id,
            "Refund ids should match"
        )

    def _get_refund_create(self, transaction):
        """
        Creates a RefundCreate payload for a given transaction.

        Args:
            transaction: The transaction to refund.

        Returns:
            RefundCreate: The refund payload.
        """
        return RefundCreate(
            transaction=transaction.id,
            type=RefundType.MERCHANT_INITIATED_ONLINE,
            external_id=str(uuid.uuid4()),
            amount=29.95
        )


    def _create(self, transaction_create):
        """
        Helper method to create a transaction.

        Args:
            transaction_create: The transaction payload.

        Returns:
            Transaction: The created transaction object.
        """
        return self.transactions_service.post_payment_transactions(
            SPACE_ID, transaction_create
        )



# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import unittest

from constants import API_CONFIG, SPACE_ID, MOCK_CARD_DATA
from utils import Utils
from postfinancecheckout.service import TransactionsService
from postfinancecheckout.models import SortingOrder, TransactionState

class TestQuerying(unittest.TestCase):
    """postfinancecheckout querying features test using transactions service"""

    def setUp(self):
        self.transactions_service = TransactionsService(API_CONFIG)

        transaction_base = self.transactions_service.post_payment_transactions(
            SPACE_ID, Utils.get_transaction_create_payload()
        )
        authorized_transaction = self.transactions_service.post_payment_transactions_id_process_card_details(
            transaction_base.id, SPACE_ID, MOCK_CARD_DATA
        )
        self.transactions_service.post_payment_transactions_id_complete_online(
            authorized_transaction.id, SPACE_ID
        )
        self.transaction1 = self.transactions_service.get_payment_transactions_id(
            authorized_transaction.id, SPACE_ID
        )

        transaction_create = Utils.get_transaction_create_payload()
        transaction_create.currency = "USD"
        transaction_create.merchant_reference = "Transaction for querying test"

        self.transaction2 = self.transactions_service.post_payment_transactions(
            SPACE_ID, transaction_create
        )

    def test_search_with_limit_should_return_correct_amount_of_items(self):
        """
        Transaction search with limit parameter set to 2 items.
        """

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=2, offset=0, order="", query=""
        )
        self.assertEqual(len(response.data), 2, "Response should contain 2 items (as limited)")

    def test_search_with_offset_should_return_correct_response(self):
        """
        Transaction search with offset parameter.
        Search selects 2 items, offset is set to 1.
        Response should contain 1 item with the higher transaction ID.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = f"id:{self.transaction1.id} OR id:{self.transaction2.id}"
        higher_id = max(self.transaction1.id, self.transaction2.id)

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=2, offset=1, order="id:ASC", query=query
        )

        self.assertEqual(response.data[0].id, higher_id, "Response offset should be correct")

    def test_search_with_before_parameter_should_return_correct_responses(self):
        """
        Transaction search using the 'before' parameter.
        Ensures transactions with IDs less than the upper bound are returned.
        """

        self.assertIsNotNone(self.transaction1, "First transaction must be initialized for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction must be initialized for this test")

        upper_id = max(self.transaction1.id, self.transaction2.id)
        lower_id = min(self.transaction1.id, self.transaction2.id)

        response = self.transactions_service.get_payment_transactions(
            SPACE_ID, before=upper_id, expand=list(), limit=10, order=SortingOrder.ASC
        )

        self.assertIsNotNone(response.data, "Response data should not be null")
        self.assertTrue(
            any(t.id == lower_id for t in response.data),
            f"Response should contain transaction with ID: {lower_id}"
        )

    def test_fetch_with_search_query_should_return_list_with_valid_items(self):
        """
        Transaction search made with query. Response should contain 2 items.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = f"id:{self.transaction1.id} OR id:{self.transaction2.id}"

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=4, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertEqual(len(response.data), 2, "Response should contain only 2 items")

    def test_fetch_with_search_query_with_quotes_should_return_list_with_valid_items(self):
        """
        Transaction search with a query containing quotes. Response should contain 1 item.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = f'merchantReference:"Transaction for querying test" AND id:{self.transaction2.id}'

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=4, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertNotEqual(len(response.data), 0, "Response should contain more than 0 items")
        self.assertEqual(len(response.data), 1, "Response should contain 1 item")

    def test_fetch_with_query_with_contains_clause_should_return_list_with_valid_items(self):
        """
        Transaction search using 'contains' clause in query. Response should contain valid items.
        """

        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = "merchantReference:~querying"

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=2, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertNotEqual(len(response.data), 0, "Response should contain more than 0 items")

    def test_fetch_with_query_with_grouping_should_return_list_with_valid_items(self):
        """
        Transaction search using a grouped query clause. Response should contain 1 item.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = f"id:{self.transaction1.id} AND (currency:EUR OR currency:CHF)"

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=4, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertEqual(len(response.data), 1, "Response should contain only 1 item")

    def test_fetch_with_query_with_range_should_return_list_with_valid_items(self):
        """
        Transaction search using range clause. Response should contain transaction with expected ID.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")

        query = f"id:>{self.transaction1.id - 2} id:<={self.transaction1.id + 2}"

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=8, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertTrue(
            any(t.id == self.transaction1.id for t in response.data),
            "Transaction with expected ID should be found"
        )

    def test_fetch_with_query_with_negation_should_return_list_with_valid_items(self):
        """
        Transaction search using negation in query.
        Response should not contain the transaction with the excluded ID.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")

        query = (
            f"id:>{self.transaction1.id - 2} id:<={self.transaction1.id + 2} "
            f"AND -id:{self.transaction1.id}"
        )

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, expand=list(), limit=8, offset=0, order="id:ASC", query=query
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertFalse(
            any(t.id == self.transaction1.id for t in response.data),
            "Transaction with excluded ID should not be present"
        )

    def test_query_with_is_null_constraint_should_return_valid_item(self):
        """
        Transaction search made with query which has isNull constraint.
        Response should contain 1 item.
        """

        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        query = f"id:{self.transaction2.id} AND paymentConnectorConfiguration:*"

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, set(), 4, 0, "id:ASC", query
        )

        self.assertIsNotNone(response, "Response should be present")
        self.assertIsNotNone(response.data)
        self.assertEqual(1, len(response.data), "Response should contain only 1 item")

    def test_search_with_desc_sorting_should_return_sorted_response(self):
        """
        Transaction search made with order clause.
        Items in response should be in descending order.
        """

        self.assertIsNotNone(self.transaction1, "First transaction should be present for this test")
        self.assertIsNotNone(self.transaction2, "Second transaction should be present for this test")

        response = self.transactions_service.get_payment_transactions_search(
            SPACE_ID, set(), 4, 0, "id:DESC", ""
        )

        self.assertIsNotNone(response)
        self.assertIsNotNone(response.data)
        self.assertGreaterEqual(len(response.data), 2, "Response should contain at least two transactions")

        first_id = response.data[0].id
        last_id = response.data[-1].id

        self.assertIsNotNone(first_id, "First transaction ID should not be None")
        self.assertIsNotNone(last_id, "Last transaction ID should not be None")

        difference = first_id - last_id

        self.assertGreater(difference, 0, "Response should be sorted descending. First ID should be greater than last ID.")

    def test_fetch_with_no_expanding_should_return_collapsed_response(self):
        """
        Gets transaction with no expand parameters.
        Checks if response does not contain any data which should be present only when expand is used.
        Example: group id should be present, but group state should be null.
        """

        self.assertIsNotNone(self.transaction1, "Transaction should be present for this test")

        response = self.transactions_service.get_payment_transactions_id(
            self.transaction1.id, SPACE_ID, set()
        )

        self.assertIsNotNone(response.group, "Group always should be present")
        self.assertIsNotNone(response.group.id, "Id always should be present")

        self.assertIsNone(response.group.state, "Group state should be null when response collapsed")
        self.assertIsNone(response.group.version, "Group version should be null when response collapsed")

    def test_fetch_with_expanding_should_return_expanded_response(self):
        """
        Gets transaction with expand parameters.
        Checks if response contains data which should be present when expand parameter is enabled.
        Example: group id and group state, version
        """

        self.assertIsNotNone(self.transaction1, "Transaction should be present for this test")
        self.assertEqual(TransactionState.FULFILL, self.transaction1.state,
                        "Transaction state should be FULFILLED for this test")

        expand_set = {"group", "paymentConnectorConfiguration"}

        response = self.transactions_service.get_payment_transactions_id(
            self.transaction1.id, SPACE_ID, expand_set
        )

        self.assertIsNotNone(response.group, "Group always should be present")
        self.assertIsNotNone(response.group.id, "Group id always should be present")

        self.assertIsNotNone(response.group.state, "Group state should be present when response expanded")
        self.assertIsNotNone(response.group.version, "Group version should be present when response expanded")

        self.assertIsNotNone(response.payment_connector_configuration)
        self.assertIsNotNone(response.payment_connector_configuration.processor_configuration,
                            "Both items in expand set should be expanded")

    def test_fetch_with_nested_expanding_should_return_expanded_response(self):
        """
        Gets transaction with nested expand parameter.
        Checks if response contains data related to parent item and to child item.
        Example: paymentConnectorConfiguration - parent item and processorConfiguration - child item.
        """

        self.assertIsNotNone(self.transaction1, "Transaction should be present for this test")
        self.assertEqual(TransactionState.FULFILL, self.transaction1.state,
                        "Transaction state should be FULFILLED for this test")

        expand_set = {"paymentConnectorConfiguration.processorConfiguration"}

        response = self.transactions_service.get_payment_transactions_id(
            self.transaction1.id, SPACE_ID, expand_set
        )

        self.assertIsNotNone(response.payment_connector_configuration)
        self.assertIsNotNone(response.payment_connector_configuration.processor_configuration)
        self.assertIsNotNone(
            response.payment_connector_configuration.processor_configuration.linked_space_id,
            "Items in nested response should be present"
        )



# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from postfinancecheckout.models.environment import Environment
from postfinancecheckout.models.failure_reason import FailureReason
from postfinancecheckout.models.label import Label
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.line_item_reduction import LineItemReduction
from postfinancecheckout.models.refund_state import RefundState
from postfinancecheckout.models.refund_type import RefundType
from postfinancecheckout.models.tax import Tax
from postfinancecheckout.models.transaction import Transaction
from typing import Optional, Set
from typing_extensions import Self

class Refund(BaseModel):
    """
    A refund is a credit issued to the customer, which can be initiated either by the merchant or by the customer as a reversal.
    """ # noqa: E501
    total_settled_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount settled for the refund, factoring in reductions, taxes, and any additional applied fees.", alias="totalSettledAmount")
    reductions: Optional[List[LineItemReduction]] = Field(default=None, description="The reductions applied on the original transaction items, detailing specific adjustments associated with the refund.")
    base_line_items: Optional[List[LineItem]] = Field(default=None, description="The original base line items from the transaction prior to the refund, serving as a reference for the refunded amounts.", alias="baseLineItems")
    processing_on: Optional[datetime] = Field(default=None, description="The date and time when the processing of the refund was started.", alias="processingOn")
    taxes: Optional[List[Tax]] = Field(default=None, description="The tax breakdown applied to the refund amount, helping with tax calculations or reporting.")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    type: Optional[RefundType] = None
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    line_items: Optional[List[LineItem]] = Field(default=None, description="The line items included in the refund, representing the reductions.", alias="lineItems")
    succeeded_on: Optional[datetime] = Field(default=None, description="The date and time when the refund succeeded.", alias="succeededOn")
    reduced_line_items: Optional[List[LineItem]] = Field(default=None, description="The line items from the original transaction, adjusted to reflect any reductions applied during the refund process.", alias="reducedLineItems")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[RefundState] = None
    merchant_reference: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The merchant's reference used to identify the refund.", alias="merchantReference")
    completion: Optional[StrictInt] = Field(default=None, description="The transaction completion that the refund belongs to.")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total monetary amount of the refund, representing the exact credit issued to the customer.")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=100)]] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    time_zone: Optional[StrictStr] = Field(default=None, description="The time zone that this object is associated with.", alias="timeZone")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    labels: Optional[List[Label]] = Field(default=None, description="The labels providing additional information about the object.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    timeout_on: Optional[datetime] = Field(default=None, description="The date and time when the object will expire.", alias="timeoutOn")
    environment: Optional[Environment] = None
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the refund was created by.", alias="createdBy")
    next_update_on: Optional[datetime] = Field(default=None, description="The date and time when the next update of the object's state is planned.", alias="nextUpdateOn")
    updated_invoice: Optional[StrictInt] = Field(default=None, description="An updated invoice reflecting adjustments made by the refund.", alias="updatedInvoice")
    failure_reason: Optional[FailureReason] = Field(default=None, alias="failureReason")
    total_applied_fees: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The sum of fees applied to the refund transaction, such as processing or service charges.", alias="totalAppliedFees")
    failed_on: Optional[datetime] = Field(default=None, description="The date and time when the refund failed.", alias="failedOn")
    transaction: Optional[Transaction] = None
    processor_reference: Optional[Annotated[str, Field(strict=True, max_length=150)]] = Field(default=None, description="The reference ID provided by the payment processor, used to trace the refund through the external payment system.", alias="processorReference")
    __properties: ClassVar[List[str]] = ["totalSettledAmount", "reductions", "baseLineItems", "processingOn", "taxes", "language", "type", "createdOn", "lineItems", "succeededOn", "reducedLineItems", "id", "state", "merchantReference", "completion", "amount", "plannedPurgeDate", "externalId", "timeZone", "version", "labels", "linkedSpaceId", "timeoutOn", "environment", "createdBy", "nextUpdateOn", "updatedInvoice", "failureReason", "totalAppliedFees", "failedOn", "transaction", "processorReference"]

    @field_validator('merchant_reference')
    def merchant_reference_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    @field_validator('external_id')
    def external_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"[	\x20-\x7e]*", value):
            raise ValueError(r"must validate the regular expression /[	\x20-\x7e]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Refund from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "total_settled_amount",
            "reductions",
            "base_line_items",
            "processing_on",
            "taxes",
            "language",
            "created_on",
            "line_items",
            "succeeded_on",
            "reduced_line_items",
            "id",
            "merchant_reference",
            "completion",
            "amount",
            "planned_purge_date",
            "external_id",
            "time_zone",
            "version",
            "labels",
            "linked_space_id",
            "timeout_on",
            "created_by",
            "next_update_on",
            "updated_invoice",
            "total_applied_fees",
            "failed_on",
            "processor_reference",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in reductions (list)
        _items = []
        if self.reductions:
            for _item_reductions in self.reductions:
                if _item_reductions:
                    _items.append(_item_reductions.to_dict())
            _dict['reductions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in base_line_items (list)
        _items = []
        if self.base_line_items:
            for _item_base_line_items in self.base_line_items:
                if _item_base_line_items:
                    _items.append(_item_base_line_items.to_dict())
            _dict['baseLineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in taxes (list)
        _items = []
        if self.taxes:
            for _item_taxes in self.taxes:
                if _item_taxes:
                    _items.append(_item_taxes.to_dict())
            _dict['taxes'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in reduced_line_items (list)
        _items = []
        if self.reduced_line_items:
            for _item_reduced_line_items in self.reduced_line_items:
                if _item_reduced_line_items:
                    _items.append(_item_reduced_line_items.to_dict())
            _dict['reducedLineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of failure_reason
        if self.failure_reason:
            _dict['failureReason'] = self.failure_reason.to_dict()
        # override the default output from pydantic by calling `to_dict()` of transaction
        if self.transaction:
            _dict['transaction'] = self.transaction.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Refund from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "totalSettledAmount": obj.get("totalSettledAmount"),
            "reductions": [LineItemReduction.from_dict(_item) for _item in obj["reductions"]] if obj.get("reductions") is not None else None,
            "baseLineItems": [LineItem.from_dict(_item) for _item in obj["baseLineItems"]] if obj.get("baseLineItems") is not None else None,
            "processingOn": obj.get("processingOn"),
            "taxes": [Tax.from_dict(_item) for _item in obj["taxes"]] if obj.get("taxes") is not None else None,
            "language": obj.get("language"),
            "type": obj.get("type"),
            "createdOn": obj.get("createdOn"),
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "succeededOn": obj.get("succeededOn"),
            "reducedLineItems": [LineItem.from_dict(_item) for _item in obj["reducedLineItems"]] if obj.get("reducedLineItems") is not None else None,
            "id": obj.get("id"),
            "state": obj.get("state"),
            "merchantReference": obj.get("merchantReference"),
            "completion": obj.get("completion"),
            "amount": obj.get("amount"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "timeZone": obj.get("timeZone"),
            "version": obj.get("version"),
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "timeoutOn": obj.get("timeoutOn"),
            "environment": obj.get("environment"),
            "createdBy": obj.get("createdBy"),
            "nextUpdateOn": obj.get("nextUpdateOn"),
            "updatedInvoice": obj.get("updatedInvoice"),
            "failureReason": FailureReason.from_dict(obj["failureReason"]) if obj.get("failureReason") is not None else None,
            "totalAppliedFees": obj.get("totalAppliedFees"),
            "failedOn": obj.get("failedOn"),
            "transaction": Transaction.from_dict(obj["transaction"]) if obj.get("transaction") is not None else None,
            "processorReference": obj.get("processorReference")
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from postfinancecheckout.models.failure_reason import FailureReason
from postfinancecheckout.models.label import Label
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.transaction import Transaction
from postfinancecheckout.models.transaction_line_item_version_state import TransactionLineItemVersionState
from typing import Optional, Set
from typing_extensions import Self

class TransactionLineItemVersion(BaseModel):
    """
    TransactionLineItemVersion
    """ # noqa: E501
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount of the updated line items, including taxes.")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    processing_on: Optional[datetime] = Field(default=None, description="The date and time when the processing of the line item version was started.", alias="processingOn")
    external_id: Optional[StrictStr] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    labels: Optional[List[Label]] = Field(default=None, description="The labels providing additional information about the object.")
    line_items: Optional[List[LineItem]] = Field(default=None, description="The line items that replace the original line items in the transaction.", alias="lineItems")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    timeout_on: Optional[datetime] = Field(default=None, description="The date and time by when the line item version is expected to be processed.", alias="timeoutOn")
    created_by: Optional[StrictInt] = Field(default=None, description="The ID of the user the line item version was created by.", alias="createdBy")
    next_update_on: Optional[datetime] = Field(default=None, description="The date and time when the next update of the line item version's state is planned.", alias="nextUpdateOn")
    failure_reason: Optional[FailureReason] = Field(default=None, alias="failureReason")
    succeeded_on: Optional[datetime] = Field(default=None, description="The date and time when the line item version was processed successfully.", alias="succeededOn")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[TransactionLineItemVersionState] = None
    linked_transaction: Optional[StrictInt] = Field(default=None, description="The payment transaction this object is linked to.", alias="linkedTransaction")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The portion of the total amount that corresponds to taxes.", alias="taxAmount")
    failed_on: Optional[datetime] = Field(default=None, description="The date and time when the processing of the line item version failed.", alias="failedOn")
    transaction: Optional[Transaction] = None
    __properties: ClassVar[List[str]] = ["amount", "plannedPurgeDate", "processingOn", "externalId", "language", "spaceViewId", "createdOn", "version", "labels", "lineItems", "linkedSpaceId", "timeoutOn", "createdBy", "nextUpdateOn", "failureReason", "succeededOn", "id", "state", "linkedTransaction", "taxAmount", "failedOn", "transaction"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TransactionLineItemVersion from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "amount",
            "planned_purge_date",
            "processing_on",
            "external_id",
            "language",
            "space_view_id",
            "created_on",
            "version",
            "labels",
            "line_items",
            "linked_space_id",
            "timeout_on",
            "created_by",
            "next_update_on",
            "succeeded_on",
            "id",
            "linked_transaction",
            "tax_amount",
            "failed_on",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of failure_reason
        if self.failure_reason:
            _dict['failureReason'] = self.failure_reason.to_dict()
        # override the default output from pydantic by calling `to_dict()` of transaction
        if self.transaction:
            _dict['transaction'] = self.transaction.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TransactionLineItemVersion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "amount": obj.get("amount"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "processingOn": obj.get("processingOn"),
            "externalId": obj.get("externalId"),
            "language": obj.get("language"),
            "spaceViewId": obj.get("spaceViewId"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "timeoutOn": obj.get("timeoutOn"),
            "createdBy": obj.get("createdBy"),
            "nextUpdateOn": obj.get("nextUpdateOn"),
            "failureReason": FailureReason.from_dict(obj["failureReason"]) if obj.get("failureReason") is not None else None,
            "succeededOn": obj.get("succeededOn"),
            "id": obj.get("id"),
            "state": obj.get("state"),
            "linkedTransaction": obj.get("linkedTransaction"),
            "taxAmount": obj.get("taxAmount"),
            "failedOn": obj.get("failedOn"),
            "transaction": Transaction.from_dict(obj["transaction"]) if obj.get("transaction") is not None else None
        })
        return _obj





# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from postfinancecheckout.models.bank_transaction import BankTransaction
from postfinancecheckout.models.line_item import LineItem
from postfinancecheckout.models.refund import Refund
from typing import Optional, Set
from typing_extensions import Self

class RefundRecoveryBankTransaction(BaseModel):
    """
    RefundRecoveryBankTransaction
    """ # noqa: E501
    line_items: Optional[List[LineItem]] = Field(default=None, description="The line items that were recovered.", alias="lineItems")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    refund_currency_value_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The value amount represents the net monetary value of the bank transaction, recorded in the refund's currency, after applicable deductions.", alias="refundCurrencyValueAmount")
    refund_currency_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The posting amount represents the monetary value of the bank transaction, recorded in the refund's currency, before applying any adjustments.", alias="refundCurrencyAmount")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    linked_transaction: Optional[StrictInt] = Field(default=None, description="The payment transaction this object is linked to.", alias="linkedTransaction")
    bank_transaction: Optional[BankTransaction] = Field(default=None, alias="bankTransaction")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    refund: Optional[Refund] = None
    __properties: ClassVar[List[str]] = ["lineItems", "linkedSpaceId", "refundCurrencyValueAmount", "refundCurrencyAmount", "language", "id", "spaceViewId", "linkedTransaction", "bankTransaction", "version", "refund"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RefundRecoveryBankTransaction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "line_items",
            "linked_space_id",
            "refund_currency_value_amount",
            "refund_currency_amount",
            "language",
            "id",
            "space_view_id",
            "linked_transaction",
            "version",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in line_items (list)
        _items = []
        if self.line_items:
            for _item_line_items in self.line_items:
                if _item_line_items:
                    _items.append(_item_line_items.to_dict())
            _dict['lineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of bank_transaction
        if self.bank_transaction:
            _dict['bankTransaction'] = self.bank_transaction.to_dict()
        # override the default output from pydantic by calling `to_dict()` of refund
        if self.refund:
            _dict['refund'] = self.refund.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RefundRecoveryBankTransaction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "lineItems": [LineItem.from_dict(_item) for _item in obj["lineItems"]] if obj.get("lineItems") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "refundCurrencyValueAmount": obj.get("refundCurrencyValueAmount"),
            "refundCurrencyAmount": obj.get("refundCurrencyAmount"),
            "language": obj.get("language"),
            "id": obj.get("id"),
            "spaceViewId": obj.get("spaceViewId"),
            "linkedTransaction": obj.get("linkedTransaction"),
            "bankTransaction": BankTransaction.from_dict(obj["bankTransaction"]) if obj.get("bankTransaction") is not None else None,
            "version": obj.get("version"),
            "refund": Refund.from_dict(obj["refund"]) if obj.get("refund") is not None else None
        })
        return _obj


